@@ -2,7 +2,8 @@ package PDF::Imposition::Schema2down;
 use strict;
 use warnings FATAL => 'all';
 
-use base qw/PDF::Imposition::Schema2up/;
+use Moo;
+extends 'PDF::Imposition::Schema2up';
 
 =head1 NAME
 
@@ -65,8 +66,25 @@ sub _do_impose {
             $gfx->formimage($rpage, 0, 0 - $self->orig_height);
         }
     }
-    $self->out_pdf_obj->saveas($self->outfile);
-    return $self->outfile;
+}
+
+=head1 INTERNALS
+
+=head2 cropmarks_options
+
+Set twoside to false.
+
+=cut
+
+sub cropmarks_options {
+    my %options = (
+                   top => 1,
+                   bottom => 1,
+                   inner => 1,
+                   outer => 1,
+                   twoside => 0,
+                  );
+    return %options;
 }
 
 1;
